"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertFilter = void 0;
const adminjs_1 = require("adminjs");
const escape_regexp_1 = __importDefault(require("escape-regexp"));
const mongoose_1 = __importDefault(require("mongoose"));
/**
 * Changes AdminJS's {@link Filter} to an object acceptible by a mongoose query.
 *
 * @param {Filter} filter
 * @private
 */
exports.convertFilter = (filter) => {
    if (!filter) {
        return {};
    }
    const filters = adminjs_1.flat.unflatten(filter.filters);
    Object.keys(filters).forEach((key) => {
        if (!Array.isArray(filters[key]))
            return;
        const reg = new RegExp(`^${escape_regexp_1.default(key)}.[0-9]+`);
        Object.keys(filter.filters).forEach(oldKey => {
            if (reg.test(oldKey))
                delete filter.filters[oldKey];
        });
        filter.filters[key] = {
            path: key,
            value: filters[key].map(f => f.value),
        };
    });
    return filter.reduce((memo, filterProperty) => {
        const { property, path, value } = filterProperty;
        if (!property && path) {
            return {
                [path]: { $in: value },
                ...memo,
            };
        }
        switch (property.type()) {
            case 'string':
                if (typeof value === 'object') {
                    if (value.equal) {
                        return {
                            [property.name()]: { $regex: `^${escape_regexp_1.default(value.equal)}$`, $options: 'i' },
                            ...memo,
                        };
                    }
                    if (value.start_with) {
                        return {
                            [property.name()]: { $regex: `^${escape_regexp_1.default(value.start_with)}`, $options: 'i' },
                            ...memo,
                        };
                    }
                    if (value.end_with) {
                        return {
                            [property.name()]: { $regex: `${escape_regexp_1.default(value.end_with)}$`, $options: 'i' },
                            ...memo,
                        };
                    }
                    if (value.contains) {
                        return {
                            [property.name()]: { $regex: escape_regexp_1.default(value.contains), $options: 'i' },
                            ...memo,
                        };
                    }
                    return {
                        [property.name()]: { $regex: '', $options: 'i' },
                        ...memo,
                    };
                }
                return {
                    [property.name()]: { $regex: escape_regexp_1.default(value), $options: 'i' },
                    ...memo,
                };
            case 'date':
            case 'datetime':
                if (value.from || value.to) {
                    return {
                        [property.name()]: {
                            ...value.from && { $gte: value.from },
                            ...value.to && { $lte: value.to },
                        },
                        ...memo,
                    };
                }
                break;
            case 'id':
                if (mongoose_1.default.Types.ObjectId.isValid(value)) {
                    return {
                        [property.name()]: value,
                        ...memo,
                    };
                }
                return {};
            default:
                break;
        }
        return {
            [property.name()]: value,
            ...memo,
        };
    }, {});
};
//# sourceMappingURL=convert-filter.js.map